/* 
   Name: spi.c
   Author: Mathäus Sander & Florian Weinhold

   Desc.: SPI function library for ATmega Prozessors
   License: LGPL
*/


#include "spi.h"
#include "../uart/uart.h"

//Basic SPI
void SPI1_Init(void) {
	/* Set MOSI, SCK  and CS output, all others input */
	DDR_SPI0 |= (1<<DD_MOSI0)|(1<<DD_SCK0)|(1<<DD_CS0);
	DDR_SPI0 &= ~(1<<DD_MISO0);

	/* Enable SPI, Master */
	SPCR = (1<<SPE)|(1<<MSTR);
	/* Set clock fck/2 */
	SPSR |= (1<<SPI2X);

#ifdef DEBUG
	printf("SPI Initialized\n");
#endif
}


//Uart1 as SPI
void SPI2_Init( void )
{
  UBRR1 = 0;
  /* Setting the XCKn port pin as output, enables master mode. */
  DDR_SPI1 |= (1<<XCK1) | (1<<DD_MOSI1);
  DDR_SPI1 &= ~(1<<DD_MISO1);
  // Set U2X mode
  UCSR1A |= (1<<U2X1);
  /* Set MSPI mode of operation and SPI data mode 0. */
  UCSR1C = (1<<UMSEL11)|(1<<UMSEL10)|(0<<UCPHA1)|(0<<UCPOL1);
  /* Enable receiver and transmitter. */
  UCSR1B = (1<<RXEN1)|(1<<TXEN1);
  /* Set baud rate. */
  /* IMPORTANT: The Baud Rate must be set after the transmitter is enabled
  */
  UBRR1 = 4;

#ifdef DEBUG
  printf("USART as SPI Initialized\n");
#endif
}

uint8_t spi1_transmit(uint8_t cData) {
	//printf("%x\n", cData);
	/* Start transmission */
	SPDR = cData;
	/* Wait for transmission complete */
	while(!(SPSR & (1<<SPIF)));

	return SPDR;
}

uint8_t spi2_transmit( uint8_t cData ){
  /* Wait for empty transmit buffer */
  while ( !( UCSR1A & (1<<UDRE1)) );
  /* Put data into buffer, sends the data */
  UDR1 = cData;
  /* Wait for data to be received */
  while ( !(UCSR1A & (1<<RXC1)) );
  /* Get and return received data from buffer */
  return UDR1;
}


